package net.p0f.samples.rhpam_fuse_integration.processes;

import org.kie.api.KieServices;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.rule.QueryResults;
import org.kie.api.runtime.rule.QueryResultsRow;

import net.p0f.samples.rhpam_fuse_integration.shared_dom.Award;
import net.p0f.samples.rhpam_fuse_integration.shared_dom.Customer;

public class ProcessWithFacts {
	public static void main(String... args) {
		KieContainer c = KieServices.Factory.get().getKieClasspathContainer();

		// Instantiate the ruleflow KB and its default session.
		KieSession s = c.getKieBase("ruleflow").newKieSession();

		Customer c1 = new Customer();
		c1.setName("John Doe");
		c1.setAge(27);
		c1.setLoyaltyLevel(Customer.GOLD);
		c1.setLoyaltyAge(10);
		c1.setMilesFlown(15000);

		Customer c2 = new Customer();
		c2.setName("Jane Doe");
		c2.setAge(35);
		c2.setLoyaltyLevel(Customer.PLATINUM);
		c2.setMilesFlown(75000);
		c2.setLoyaltyAge(10);

		Award a1 = null;
		Award a2 = null;

		// option 1: insert facts into working memory (INVISIBLE TO PROCESS)
		s.insert(c1);
		s.insert(c2);
		s.insert(a1);
		s.insert(a2);

		// Process ID does not have to follow package naming convention.
		//
		// HOWEVER: While rules are loaded regardless of their location in the
		// filesystem, and the "package" keyword decides which KBs will see them or not,
		// process models MUST be in the correct location in the filesystem (iow, if the
		// directory path does not equal the package of the process, including
		// underscores in package names, the KieContainer will not load it into the KB).
		s.startProcess("ruleflow.RulesRulesRules");
		s.fireAllRules();

		System.out.println("******** Examining Working Memory ********");

		// Note that the rule engine sees the customers, awards are given, even the
		// "Rule" language condition in the diverging gateway matches, but the last
		// task can't see a single customer to congratulate, because it's trying to use
		// a process variable, which is not what was modified by the rules.
		QueryResults qr = s.getQueryResults("Get Awards");
		for (QueryResultsRow qrr : qr) {
			Award award = (Award) qrr.get("award");
			System.out.println("Found award: " + award.toString());
		}

		s.dispose();
	}
}
